import createNotImplementedUseCase from './createNotImplementedUseCase.js';
export default (function (_ref) {
  var useCases = _ref.useCases,
    config = _ref.config,
    logger = _ref.logger,
    pde = _ref.pde;
  return /*#__PURE__*/function () {
    function EntryPoint(params) {
      if (params === void 0) {
        params = {
          config: {},
          logger: {},
          pde: {}
        };
      }
      this.subscribers = {};
      // decide to use a static config from the factory
      // or use a config passed to the constructor that could be mutated
      this._config = config || params.config;
      this._useCases = useCases;
      this._logger = logger || params.logger;
      this._pde = pde || params.pde;
    }

    /**
     * Retrieves one entry from the public dictionary
     * The dictionary will contain the use cases and configuration
     * @example
     * const configuration = domain.get('config')
     * const useCase = domain.get('phone_contact_use_case')
     * @param {string} key The name of the use case or indicate that we want to get the config
     * @return {function|{execute: Function, $: object}|NotImplementedUseCase}
     */
    var _proto = EntryPoint.prototype;
    _proto.get = function get(key) {
      var _this = this;
      // config key is still sync and we should return as it
      if (key === 'config') {
        return this._config;
      }
      // get the useCase using the key passed by the user
      var useCase = this._useCases[key];
      // if the useCase doesn't exist, then let developer know that the useCase is not implemented
      if (typeof useCase === 'undefined') return createNotImplementedUseCase(key);
      var isDynamicImportWholeFactory = useCase instanceof Array;
      var _ref2 = isDynamicImportWholeFactory ? useCase // for the whole factory we extract the single method from the array
        : [useCase],
        loader = _ref2[0],
        method = _ref2[1]; // for the single factory, the method is undefined as is default

      var getMethod = isDynamicImportWholeFactory ? function (factory) {
        return factory.default[method];
      } : function (factory) {
        return factory.default;
      };

      // if loader is undefined then is not implemented, otherwhise load async the useCase
      return loader === undefined ? createNotImplementedUseCase(key) : {
        execute: function execute(params) {
          var subscriptionsForUseCase = _this.subscribers[key] || [];
          // load async the factory, execute use case and return the promise
          return loader().then(function (factory) {
            return getMethod(factory)({
              config: _this._config,
              logger: _this._logger,
              pde: _this._pde
            }).execute(params);
          }).then(function (result) {
            subscriptionsForUseCase.forEach(function (fn) {
              return fn({
                error: null,
                params: params,
                result: result
              });
            });
            return result;
          }).catch(function (e) {
            subscriptionsForUseCase.forEach(function (fn) {
              return fn({
                error: e,
                params: params,
                result: null
              });
            });
            return Promise.reject(e);
          });
        },
        subscribe: function subscribe(callback) {
          _this.subscribers[key] = _this.subscribers[key] || [];
          _this.subscribers[key].push(callback);

          // return a way to remove the listener
          return {
            unsubscribe: function unsubscribe() {
              _this.subscribers[key] = _this.subscribers[key].filter(function (l) {
                return l !== callback;
              });
            }
          };
        },
        $: {
          // support deprecated decorator @streamify
          execute: {
            subscribe: function subscribe(onNext, onError) {
              // creating an object here that will have a dispose method
              var ret = {
                dispose: function dispose() {}
              };
              loader().then(function (factory) {
                // black magic: mutate the object, very small memory leak but that
                // makes dispose working async and we need it
                ret.dispose = getMethod(factory)({
                  config: _this._config,
                  logger: _this._logger,
                  pde: _this._pde
                }).$.execute.subscribe(onNext, onError).dispose;
              });
              // return the object that will be mutated async
              return ret;
            }
          }
        }
      };
    }

    /**
     * Set a value to a key in the config
     * @example
     * const configuration = domain.config('new_config', 'new value')
     * @param {String} key
     * @param {String} value
     * @return {this}
     */;
    _proto.config = function config(key, value) {
      this._config.set(key, value);
      return this;
    };
    return EntryPoint;
  }();
});